//*****************************************
// Photo Box with ATmega2560
// CPU Clock:O16MHz Ext,Crystal 16MHz)
//   iq[Yf[^@WȂŊOUɂj

// File Name:Mega_Twi.c
//
//  PC5(SCL): TWI-SCL
//  PC4(SDA): TWI-SDA
//
//  C:\WinAVR-20100110\doc\avr-libc\avr-libc-user-manual.pdf ɊeCN[ht@C̐
//
//  2012/06/13 ishi New
//  2023/03/07 Modify for mega328_i2c_old Taishi
//  2023/05/13 Modify for mega328_iic_oled_rtc Taishi
//  2023/12/09 Modify for Photo Box with ATmega2560 Taishi
//*****************************************

#include	<avr/io.h>
#include	<compat/twi.h>

#include	"pr_mega2560_photo_box_h.h"

#include	"port_bit_h.h"
#include	"mega_twi_h.h"

#define MAX_TRIES 50

unsigned char	gTwi_status;


// ************************************
// TWI ݒ
//   @Ȃ
//  ߂l Ȃ
// ************************************
void	twi_init(void)
{

	TWBR = 32;						// NbN߂
	TWSR = 0;						// TWSR ̕ 00 1/1Ƃ̂ 16MHz/(16+2*32*1) = 200kHz
									// botNbNPUlg̎́@200gɂȂ
}

// ************************************
// TWI ԗv
//   typeFTWI_START, TWI_DATA, TWI_DATA_ACK or TWI_STOP
//  ߂lFTWI (TWSR D3--D7)
//  ʂ҂Ĕ
// ************************************
static
unsigned char twi_transmit(unsigned char type)
{
	switch(type) {
	  case TWI_START: 										// Jnԗv
		TWCR = ( (1 << TWINT) | (1 << TWSTA) | (1 << TWEN) );
		break;
	  case TWI_DATA: 										// ŏ
		TWCR = ( (1 << TWINT) | (1 << TWEN) );
		break;
	  case TWI_DATA_ACK: 									// Lŏi]uŎg݂Hj
		TWCR = ( (1 << TWEA) | (1 << TWINT) | (1 << TWEN) );
		break;
	  case TWI_STOP: 										// ~ԗv
		TWCR = ( (1 << TWINT) | (1 << TWEN) | (1 << TWSTO) );
		return(0);
	}

	while (!(TWCR & (1 << TWINT))){							// TWINT-bit PɂȂ܂ő҂
		;
	}

	return (TWSR & 0xF8);									// TWI Ԃ߂
}

// ************************************
// TWI Jn
//    dev_addr: foCXAhX
//        rw_type: TW_WRITE or TW_READ
//  ߂lF0= JnԂɂȂB 1=JnԂɂȂȂ
//
//  ʂ҂Ĕ
// ************************************
unsigned char	twi_start(unsigned int dev_addr, unsigned char rw_type)
{
	unsigned char	n;
	unsigned char	r_val = 1;

	for(n = 0; n < MAX_TRIES; n++){
		gTwi_status=twi_transmit(TWI_START);								// Jnԗv
		if (gTwi_status == TW_MT_ARB_LOST){									// arbitration lost in SLA+W or data ̎
			continue;														// gC
		}
		if ( ((gTwi_status != TW_START) && (gTwi_status != TW_REP_START)) ){	// start condition transmitted ŖāA repeated start condition transmitted ł
			break;
		}
		TWDR = ( (dev_addr & 0xfe) | rw_type);								// ǂ  Ɂ@foCXhcƃfoCXAhXt f[^WX^ɃZbg

		gTwi_status=twi_transmit(TWI_DATA);									//  Mv

		if(rw_type == TW_WRITE){
			if ( ((gTwi_status == TW_MT_SLA_NACK) || (gTwi_status == TW_MT_ARB_LOST)) ){	// SLA+W transmitted, NACK received A arbitration lost in SLA+W or data ̎
				continue;														// gC
			}

			if (gTwi_status != TW_MT_SLA_ACK){									// SLA+W transmitted, ACK received Ŗ
				;
			}else{
				r_val=0;														// ߂lOɂ
			}
			break;																// [v𔲂
		}else{
			if ( ((gTwi_status == TW_MR_SLA_NACK) || (gTwi_status == TW_MR_ARB_LOST)) ){	// SLA+R transmitted, NACK received A arbitration lost in SLA+R or data ̎
				continue;														// gC
			}

			if (gTwi_status != TW_MR_SLA_ACK){									// SLA+R transmitted, ACK received Ŗ
				;
			}else{
				r_val=0;														// ߂lOɂ
			}
			break;																// [v𔲂
		}
	}

	return(r_val);
}

// ************************************
// TWI ~
//     Ȃ
//  ߂l Ȃ
//
// ************************************
void twi_stop(void)
{
	gTwi_status=twi_transmit(TWI_STOP);	// ~v
}

// ************************************
// TWI 
//     data: ݃f[^
//  ߂lF0= ߂B 1=߂Ȃ
//
//  ʂ҂Ĕ
// ************************************
unsigned char	twi_write(unsigned char data)
{
	unsigned char	r_val = 1;

	TWDR = data;							// ݃f[^ f[^WX^ɃZbg

	gTwi_status=twi_transmit(TWI_DATA);		//  Mv

	if (gTwi_status == TW_MT_DATA_ACK){		// data transmitted, ACK received ̎
		r_val=0;
	}
	return(r_val);
}

// ************************************
// TWI ǂݏo
//     data: ǂݏof[^鏊
//     ack_type: 1:LA 0:
//  ߂lF0= ǂݏoB 1=ǂݏoȂ
//
//  ʂ҂Ĕ
// ************************************
unsigned char	twi_read(unsigned char *data,char ack_type)
{
	unsigned char	r_val = 1;

	if (ack_type) {										// L w̎
		gTwi_status=twi_transmit(TWI_DATA_ACK);			// L f[^ǂݏo
		if (gTwi_status == TW_MR_DATA_ACK){				// data transmitted, ACK received ̎
			*data=TWDR;									// f[^ǂݏoĎw̏ɓ
			r_val=0;
		}
	} else {
		gTwi_status=twi_transmit(TWI_DATA);				//  f[^ǂݏo
		if (gTwi_status == TW_MR_DATA_NACK){				// data transmitted, NACK received ̎
			*data=TWDR;									// f[^ǂݏoĎw̏ɓ
			r_val=0;
		}
	}

	return(r_val);
}

